# Copyright (C) 2005 Canonical Limited
#   Authors: Robert Collins <robert.collins@canonical.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from bzrlib.selftest.TestUtil import TestLoader, TestSuite
from bzrlib.selftest import TestCaseInTempDir
try:
    import pybaz
except ImportError:
    pybaz = None
import os
from baz_import import import_version, revision_id
from bzrlib.branch import find_branch

def test_suite():
    if pybaz is None:
        return TestSuite()
    return TestLoader().loadTestsFromName(__name__)
    

class TestBazImport(TestCaseInTempDir):

    def setUp(self):
        TestCaseInTempDir.setUp(self)
        self._homedir = os.path.join(os.getcwdu(), 'home')
        self._oldhome = os.environ['HOME']
        os.mkdir(self._homedir)
        os.environ['HOME'] = self._homedir
        self._archiveroot = os.path.join(os.getcwdu(), 'archive')
        self._archive = pybaz.make_archive('demo@DONOTUSE', str(self._archiveroot))
        os.mkdir('tree')
        pybaz.set_my_id("Test User<test@example.org>")
        tree = pybaz.init_tree('tree', 'demo@DONOTUSE/c--import--0')
        msg = tree.log_message()
        msg["summary"] = "I am importing now"
        tree.import_(msg)

        self._empty_tag = 'demo@DONOTUSE/c--empty-tag--0'
        self._empty_tag_bzr = revision_id(self._empty_tag + '--base-0')
        pybaz.Revision('demo@DONOTUSE/c--import--0--base-0').make_continuation(
            pybaz.Version(self._empty_tag))

        self._empty_merged_tag = 'demo@DONOTUSE/c--empty-merged-tag--0'
        self._empty_merged_tag_bzr_base = revision_id(self._empty_merged_tag 
                                                 + '--base-0')
        self._empty_merged_tag_bzr = revision_id(self._empty_merged_tag 
                                                 + '--patch-1')
        pybaz.Revision('demo@DONOTUSE/c--import--0--base-0').make_continuation(
            pybaz.Version(self._empty_merged_tag))
        tree = pybaz.Revision(self._empty_merged_tag + '--base-0').get(
            'tree-empty-merged')
        tree.star_merge(self._empty_tag,
                        pybaz.Version('demo@DONOTUSE/c--import--0'))
        msg = tree.log_message()
        msg["summary"]="did a merge, yarh"
        tree.commit(msg)

    def tearDown(self):
        os.environ['HOME'] = self._oldhome
        
    def test_import_empty(self):
        import_version('output', pybaz.Version('demo@DONOTUSE/c--import--0'))
        # expected results:
        # one commit, no files, revision identifier of 'demo@DONOTUSE_c--import--0--base-0'
        branch = find_branch('output', find_root=False)
        self.assertEqual(branch.revision_history(),
                         ['Arch-1:demo@DONOTUSE%c--import--0--base-0'])
        rev = branch.get_revision('Arch-1:demo@DONOTUSE%c--import--0--base-0')
        # and again.
        import_version('output2', pybaz.Version('demo@DONOTUSE/c--import--0'))
        branch2 = find_branch('output2', find_root=False)
        self.assertEqual(branch.revision_history(), branch2.revision_history())
        rev2 = branch2.get_revision('Arch-1:demo@DONOTUSE%c--import--0--base-0')
        # they must be the same
        self.assertEqual(rev, rev2)

        # and we should get some expected values:
        self.assertEqual(rev.committer, "Test User<test@example.org>")
        self.assertEqual(rev.message, "I am importing now")
        self.assertEqual(rev.revision_id,
                         "Arch-1:demo@DONOTUSE%c--import--0--base-0")

    def test_empty_tagged(self):
        import_version('output', pybaz.Version(self._empty_tag))
        # expected results:
        # two commits, no files, revision identifiers of 
        # 'demo@DONOTUSE_c--import--0--base-0' and
        # self._empty_tag_bzr
        branch = find_branch('output', find_root=False)
        self.assertEqual(branch.revision_history(),
                         ['Arch-1:demo@DONOTUSE%c--import--0--base-0',
                          self._empty_tag_bzr])
        rev = branch.get_revision(self._empty_tag_bzr)
        # and again.
        import_version('output2', pybaz.Version(self._empty_tag))
        branch2 = find_branch('output2', find_root=False)
        self.assertEqual(branch.revision_history(), branch2.revision_history())
        rev2 = branch2.get_revision(self._empty_tag_bzr)
        # they must be the same
        self.assertEqual(rev, rev2)

        # and we should get some expected values:
        self.assertEqual(rev.committer, "Test User<test@example.org>")
        self.assertEqual(rev.message, "tag of demo@DONOTUSE/c--import--0--base-0")
        self.assertEqual(rev.revision_id, self._empty_tag_bzr)

    def test_empty_merged_tagged(self):
        import_version('output', pybaz.Version(self._empty_merged_tag))
        # expected results:
        # two commits, no files, revision identifiers of 
        # 'demo@DONOTUSE_c--import--0--base-0' and
        # self._empty_merged_tag_bzr_base
        # self._empty_merged_tag_bzr
        # and a merged revision from the latter of
        # self._empty_tag_bzr
        branch = find_branch('output', find_root=False)
        self.assertEqual(branch.revision_history(),
                         ['Arch-1:demo@DONOTUSE%c--import--0--base-0',
                          self._empty_merged_tag_bzr_base,
                          self._empty_merged_tag_bzr])
        # and again.
        import_version('output2', pybaz.Version(self._empty_merged_tag))
        branch2 = find_branch('output2', find_root=False)
        # and import what we should be merged up against for checking with.
        import_version('output3', pybaz.Version(self._empty_tag))
        branch3 = find_branch('output3', find_root=False)
        
        self.assertEqual(branch.revision_history(), branch2.revision_history())
        self.assertNotEqual(branch.revision_history(), branch3.revision_history())
        # check revisions in the history.
        rev = branch.get_revision(self._empty_merged_tag_bzr_base)
        rev2 = branch2.get_revision(self._empty_merged_tag_bzr_base)
        # they must be the same
        self.assertEqual(rev, rev2)
        # and we should get some expected values:
        self.assertEqual(rev.committer, "Test User<test@example.org>")
        self.assertEqual(rev.message, "tag of demo@DONOTUSE/c--import--0--base-0")
        self.assertEqual(rev.revision_id, self._empty_merged_tag_bzr_base)

        # check next revisions in the history.
        rev = branch.get_revision(self._empty_merged_tag_bzr)
        rev2 = branch2.get_revision(self._empty_merged_tag_bzr)
        # they must be the same
        self.assertEqual(rev, rev2)
        # and we should get some expected values:
        self.assertEqual(rev.committer, "Test User<test@example.org>")
        self.assertEqual(rev.message, "did a merge, yarh")
        self.assertEqual(rev.revision_id, self._empty_merged_tag_bzr)
        self.assertEqual(rev.parents[0].revision_id,
                         self._empty_merged_tag_bzr_base)
        self.assertEqual(rev.parents[1].revision_id,
                         self._empty_tag_bzr)

        # this tree should have nothing missing from that tree.   
        # FIXME there is no code for this right now.
        # self.assertEqual(branch.missing_revisions(branch3), [])
